//
// MyView.m
// Core Animation Screen Cast
//
// Created by Bill Dudney on 05/01/08.
//
// Copyright 2008 Gala Factory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#import "MyView.h"
#import <QuartzCore/QuartzCore.h>
#import "CGColorHolder.h"
#import "MyDelegate.h"
#import "MyLayer.h"

@interface MyView (Private)

- (void)addRotationToLayer:(CALayer *)layer;
- (CABasicAnimation *)customAnimation;
- (CGImageRef)loadImage;
- (CGRect)bestFitForImage:(CGImageRef) image;

@end

@implementation MyView

- (void)awakeFromNib {
	self.layer = [CALayer layer];
	self.wantsLayer = YES;
	self.layer.backgroundColor = [CGColorHolder black];
	CALayer *sublayer = [CALayer layer];
	sublayer.contents = (id)[self loadImage];
	sublayer.bounds = CGRectMake(0.0f, 0.0f, NSWidth(self.bounds) * 0.5f, 
								 NSHeight(self.bounds) * 0.5f);
	sublayer.position = CGPointMake(NSMidX(self.bounds), NSMidY(self.bounds));
	sublayer.delegate = [[MyDelegate alloc] init];
	[self.layer addSublayer:sublayer];
}

- (BOOL)acceptsFirstResponder {
	return YES;
}

- (void)moveLeft:(id)sender {
	CALayer *sublayer = [[self.layer sublayers] objectAtIndex:0];
	CGPoint newPosition = 
	CGPointMake(sublayer.anchorPoint.x * CGRectGetWidth(sublayer.bounds), 
				NSMidY(self.bounds));
	[sublayer setPosition:newPosition];
	[self addRotationToLayer:sublayer];
}


- (void)moveRight:(id)sender {
	CALayer *sublayer = [[self.layer sublayers] objectAtIndex:0];
	CGPoint newPosition = 
	CGPointMake(NSWidth([self bounds]) - (sublayer.anchorPoint.x * CGRectGetWidth(sublayer.bounds)), 
				NSMidY(self.bounds));
	[sublayer setPosition:newPosition];
}

- (void)moveUp:(id)sender {
	CALayer *sublayer = [[self.layer sublayers] objectAtIndex:0];
	CGPoint newPosition = 
	CGPointMake(NSMidX(self.bounds), 
				NSHeight(self.bounds) - sublayer.anchorPoint.y * CGRectGetHeight(sublayer.bounds));
	[sublayer setPosition:newPosition];
}

- (void)moveDown:(id)sender {
	CALayer *sublayer = [[self.layer sublayers] objectAtIndex:0];
	CGPoint newPosition = 
	CGPointMake(NSMidX(self.bounds), 
				sublayer.anchorPoint.y * CGRectGetHeight(sublayer.bounds));
	[sublayer setPosition:newPosition];
}

@end

@implementation MyView (Private)

- (void)addRotationToLayer:(CALayer *)layer {
	CAKeyframeAnimation *rotation = [CAKeyframeAnimation animation];
	rotation.values = [NSArray arrayWithObjects:
					   [NSValue valueWithCATransform3D:CATransform3DMakeRotation(0.0f, 0.0f, 1.0f, 0.0f)],
					   [NSValue valueWithCATransform3D:CATransform3DMakeRotation(M_PI, 0.0f, 1.0f, 0.0f)],
					   [NSValue valueWithCATransform3D:CATransform3DMakeRotation(2.0f * M_PI, 0.0f, 1.0f, 0.0f)],nil];
	rotation.duration = 2.0f;
	[layer addAnimation:rotation forKey:@"transform"];
}

- (CABasicAnimation *)customAnimation {
	CABasicAnimation *animation = [CABasicAnimation animation];
	animation.duration = 2.0f;
	animation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
	return animation;
}

- (CGRect) bestFitForImage:(CGImageRef)image {
	CGSize size = CGSizeMake(NSWidth([self bounds]), NSHeight([self bounds]));
	CGFloat imageWidth = CGImageGetWidth(image);
	CGFloat imageHeight = CGImageGetHeight(image);
	CGFloat scale = 0.0f;
	if(size.width / imageWidth < size.height / imageHeight) {
		scale = size.width / imageWidth;
	} else {
		scale = size.height / imageHeight;
	}
	CGRect rect = {CGPointZero, {scale * imageWidth, scale * imageHeight}};
	// if we have a portrait image we want to center it on the width
	if(rect.size.width < size.width) {
		rect.origin.x = (size.width - rect.size.width)/2.0f;
	}
	return rect;
}

- (CGImageRef)loadImage {
	CGImageRef image = NULL;
	NSBundle *bun = [NSBundle mainBundle];
	NSString *path = [bun pathForResource:@"GoldenGateBridge" ofType:@"jpg"];
	NSURL * url = [NSURL fileURLWithPath:path];
	CGImageSourceRef imageSource = CGImageSourceCreateWithURL((CFURLRef)url, NULL);
	if(imageSource != NULL) {
		image = CGImageSourceCreateImageAtIndex(imageSource, 0, NULL);
		CFRelease(imageSource);
	} else {
		NSLog(@"Could not load image");
		return NULL;
	}
	
	if(NULL != image) {
		CGRect rect = [self bestFitForImage:image];
		CGColorSpaceRef colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
		CGContextRef context = CGBitmapContextCreate(NULL, rect.size.width, 
													 rect.size.height, 8, 0, 
													 colorSpace, 
													 kCGImageAlphaPremultipliedFirst); 
		CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
		rect.origin = CGPointZero;
		CGContextDrawImage(context, rect, image);
		CGContextFlush(context);
		CGImageRef scaledImage = CGBitmapContextCreateImage(context);
		CGContextRelease (context);
		CGImageRelease(image);
		image = NULL;
		image = scaledImage;
	}
	
	return image;
}

@end
